# Hermes Matlab Bindings


## Requirements
- Pre-installed Hermes drivers, please refer to the Hermes SDK documentation
- Matlab 2017a+

## Supported architectures and platforms
Currently only the x86_64 (i.e. AMD64) architecture on both Linux and Windows is supported.
For additional architectures and/or platforms please contact Micro Photon Devices.

## Introduction
This directory contains the Matlab bindings for the Hermes camera SDK and some test/example scripts.
- <em>Hermes.m</em> defines the Hermes wrapping all hermes SDK functions
- <em>HermesExamples.m</em> is a collection of self-contained example scripts
- the <em>lib</em> directory contains the Hermes SDK libraries (.dll for Windows, .so for linux)

## Getting Started
Connect and power up the Hermes Camera.
A quick test of the camera in <em>Live</em> can be performed by running the <em>Live</em> function from the <em>HermesExamples.m</em> file.

In the Matlab Command Window:
```matlab
HermesExamples.Live()
```
This should display a series of acquired live frames like below:

![live frame example image](docs/live_frame_example.png "live frame")

## Additional Examples
Other examples implementing almost all Hermes functionalities can be called from the <em>HermesExamples.m</em> file.

In the Matlab Command Window or in a new .m script:
```matlab
% run the Live mode example
HermesExamples.Live()
% run the subarray mode test
HermesExamples.LiveSubArray()
% run the background subtraction example
HermesExamples.BackgroundSubtraction()
% run the snap acquisition mode example
HermesExamples.Snap()
% run the continuous acquisition on file example
HermesExamples.ContinuousFile()
% run the continuous acquisition in memory example
HermesExamples.ContinuousInMemory()
% run the FLim mode example
HermesExamples.Flim()
```
The content of all example functions in <em> HermesExamples.m</em> can be used as a starting point for other user scripts.